@@ -294,15 +294,15 @@ static int perf_init() {
             if(unlikely(fd < 0)) {
                 switch errno {
                     case EACCES:
-                        error("Cannot access to the PMU: Permission denied");
+                        collector_error("Cannot access to the PMU: Permission denied");
                         break;
                     case EBUSY:
-                        error("Another event already has exclusive access to the PMU");
+                        collector_error("Another event already has exclusive access to the PMU");
                         break;
                     default:
-                        error("Cannot open perf event");
+                        collector_error("Cannot open perf event");
                 }
-                error("Disabling event %u", current_event->id);
+                collector_error("Disabling event %u", current_event->id);
                 current_event->disabled = 1;
             }
 
@@ -346,7 +346,7 @@ static void reenable_events() {
             if(ioctl(current_fd, PERF_EVENT_IOC_DISABLE, PERF_IOC_FLAG_GROUP) == -1
                || ioctl(current_fd, PERF_EVENT_IOC_ENABLE, PERF_IOC_FLAG_GROUP) == -1)
             {
-                error("Cannot reenable event group");
+                collector_error("Cannot reenable event group");
             }
         }
     }
@@ -388,7 +388,7 @@ static int perf_collect() {
                 current_event->updated = 1;
             }
             else {
-                error("Cannot update value for event %u", current_event->id);
+                collector_error("Cannot update value for event %u", current_event->id);
                 return 1;
             }
         }
@@ -520,8 +520,8 @@ static void perf_send_metrics() {
                , "instructions_per_cycle"
                );
 
-        calculated_number result = ((calculated_number)perf_events[EV_ID_INSTRUCTIONS].value /
-                                    (calculated_number)perf_events[EV_ID_CPU_CYCLES].value) * 100.0;
+        NETDATA_DOUBLE result = ((NETDATA_DOUBLE)perf_events[EV_ID_INSTRUCTIONS].value /
+                                    (NETDATA_DOUBLE)perf_events[EV_ID_CPU_CYCLES].value) * 100.0;
         printf("SET %s = %lld\n"
                , "ipc"
                , (collected_number) result
@@ -1272,11 +1272,11 @@ void parse_command_line(int argc, char **argv) {
             exit(1);
         }
 
-        error("ignoring parameter '%s'", argv[i]);
+        collector_error("ignoring parameter '%s'", argv[i]);
     }
 
     if(!plugin_enabled){
-        info("no charts enabled - nothing to do.");
+        collector_info("no charts enabled - nothing to do.");
         printf("DISABLE\n");
         exit(1);
     }
@@ -1284,18 +1284,7 @@ void parse_command_line(int argc, char **argv) {
 
 int main(int argc, char **argv) {
     clocks_init();
-
-    // ------------------------------------------------------------------------
-    // initialization of netdata plugin
-
-    program_name = "perf.plugin";
-
-    // disable syslog
-    error_log_syslog = 0;
-
-    // set errors flood protection to 100 logs per hour
-    error_log_errors_per_period = 100;
-    error_log_throttle_period = 3600;
+    nd_log_initialize_for_external_plugins("perf.plugin");
 
     parse_command_line(argc, argv);
 
@@ -1304,7 +1293,7 @@ int main(int argc, char **argv) {
     if(freq >= update_every)
         update_every = freq;
     else if(freq)
-        error("update frequency %d seconds is too small for PERF. Using %d.", freq, update_every);
+        collector_error("update frequency %d seconds is too small for PERF. Using %d.", freq, update_every);
 
     if(unlikely(debug)) fprintf(stderr, "perf.plugin: calling perf_init()\n");
     int perf = !perf_init();
@@ -1327,7 +1316,7 @@ int main(int argc, char **argv) {
         if(unlikely(netdata_exit)) break;
 
         if(unlikely(debug && iteration))
-            fprintf(stderr, "perf.plugin: iteration %zu, dt %llu usec\n"
+            fprintf(stderr, "perf.plugin: iteration %zu, dt %"PRIu64" usec\n"
                     , iteration
                     , dt
             );
@@ -1348,6 +1337,6 @@ int main(int argc, char **argv) {
         if(now_monotonic_sec() - started_t > 14400) break;
     }
 
-    info("process exiting");
+    collector_info("process exiting");
     perf_free();
 }
